#!/usr/bin/env python
# -*- coding: utf-8 -*-

#isinstance(l0,gimp.Image)

import gimp, gimpplugin, math
from gimpenums import *
pdb = gimp.pdb
import gtk, gimpui, pnt2d
from pnt2d import *

import os, sys
import gettext
p=os.path.dirname(sys.argv[0]) + os.sep +'locales'
gettext.install("arakne-path-shape-creator", p, unicode=True)

SHAPES=[[_('Kwadrat'),0],[_('Prostokąt'),1],[_('Krzyż'),5],[_('Romb'),4],[_('Wielokąt'),2],[_('Gwiazda'),3]]
CORNERS=[[_('Zwykłe'),0],[_('Ścięte'),1],[_('Zaokrąglone'),2],[_('Wklęsłe'),3]]

def binomial(i, n):
	return math.factorial(n) / float(math.factorial(i) * math.factorial(n - i))

def bernstein(t, i, n):
	return binomial(i, n) * (t ** i) * ((1 - t) ** (n - i))

def bezier(t, points):
	n = len(points) - 1
	x = y = 0
	for i, pos in enumerate(points):
		bern = bernstein(t, i, n)
		x += pos[0] * bern
		y += pos[1] * bern
	return (x, y)

def pntAtDist(p1,p2,dist):
	p3= pnt2d(p2.x - p1.x,p2.y - p1.y)
	length = p3.length()
	p4=pnt2d(p3.x/length,p3.y/length)
	p4.scale(dist,dist)
	return pnt2d(p1.x+p4.x, p1.y+p4.y)

def angBetween2Lines(pC,p1,p2): # pC punto comun
	return math.atan2(pC.y-p1.y, pC.x-p1.x) - math.atan2(pC.y-p2.y, pC.x-p2.x)

def createSmallArcBez(r, a1, a2):
	a = (a2 - a1) / 2.0
	p4 = pnt2d(r * math.cos(a), r * math.sin(a))
	p1 = pnt2d(p4.x, -p4.y)
	k = 0.5522847498
	f = k * math.tan(a)
	p2 = pnt2d(p1.x + f * p4.y, p1.y + f * p4.x)
	p3 = pnt2d(p2.x,-p2.y) 
	ar = a + a1
	P1 = pnt2d(r * math.cos(a1), r * math.sin(a1)) 
	P2 = pnt2d(p2.x, p2.y).rotate(ar)
	P3 = pnt2d(p3.x, p3.y).rotate(ar)
	P4 = pnt2d(r * math.cos(a2),r * math.sin(a2))
	B1=bezpnt(P1,None,P2)
	B2=bezpnt(P4,P3)
	return [B1,B2]

EPSILON = 0.00001

def createArcBez(rad, sAng, eAng, posY=0.0):
	verts=[]
	bezs =[]
	pi2 = math.pi * 2
	sAng,eAng = (sAng % pi2, eAng % pi2)
	piOver2 = math.pi / 2.0
	sign = 1 if (sAng < eAng) else -1
	a1 = sAng
	totAng = min(math.pi*2, abs(eAng - sAng))
	while (totAng > EPSILON):
		a2 = a1 + sign * min(totAng, piOver2)
		bezs.extend(createSmallArcBez(rad, a1, a2))
		totAng = totAng - abs(a2 - a1)
		a1 = a2
	return bezs

def rotArcBez(bezs,ang):
	pts2=[]
	rrr = math.radians(ang)
	for bez in bezs:
		bp=bezpnt().copy(bez).rotate(rrr)
		pts2.extend([bp])
	return pts2

def arcLinesBez(pC,P1,P2,rad):
	tBezs = []
	Rot = angBetween2Lines(pC,P1,P2)
	hipo=rad/math.sin(Rot/2)
	catB=rad/math.tan(Rot/2)
	bz1 = pntAtDist(pC,P1,catB)
	bx1 = pntAtDist(pC,P2,catB)
	pC2 = pnt2d(0,-hipo+pC.y)
	Rot2 = angBetween2Lines(pC2,bx1,bz1)
	iniRot=(math.pi-Rot2)/2.0
	tBezs = createArcBez(rad, iniRot, Rot2+iniRot)
	for bez in tBezs:
		bez.translate(0.0,pC.y-abs(hipo))
	return tBezs

def polyRoundBez(rad,rot,sides,rad2):
	bezs=[]
	rrot=math.radians(rot)
	pC=pnt2d(0,rad)
	if rad2 == 0.0 :
		for n in range(int(sides)):
			bezs.append(bezpnt(pnt2d(pC.x,pC.y)))
			pC.rotate(rrot)
	else:
		P1=pnt2d(0,rad).rotate(rrot)
		P2=pnt2d(0,rad).rotate(-rrot)
		bez = arcLinesBez(pC,P1,P2,rad2)
		for rr in range(int(sides)):
			bezs.extend(rotArcBez(bez,-rot*rr))
	return bezs

def starRoundBez(rad,radInt,rot,sides,rad2):
	bezs=[]
	rrot=math.radians(rot)
	pC=pnt2d(0,rad)
	pC2=pnt2d(0,radInt)
	if rad2 == 0.0 :
		pC2.rotate(rrot)
		for n in range(int(sides/2)):
			bezs.append(bezpnt(pnt2d(pC.x,pC.y)))
			bezs.append(bezpnt(pnt2d(pC2.x,pC2.y)))
			pC.rotate(rrot*2)
			pC2.rotate(rrot*2)
	else:
		P1 = pnt2d(0,radInt).rotate(rrot)
		P2 = pnt2d(0,radInt).rotate(-rrot)
		pts = arcLinesBez(pC,P1,P2,rad2)
		pC2.rotate(-rrot)
		for rr in range(int(sides)):
			if rr % 2 == 0:
				bezs.extend(rotArcBez(pts,-rot*rr))
			else:
				bezs.append(bezpnt(pnt2d(pC2.x,pC2.y)))
				pC2.rotate(-rrot*2)
	return bezs

def bezier_curve_range(n, points):
	"""Range of points in a curve bezier"""
	for i in xrange(n):
		t = i / float(n - 1)
		yield bezier(t, points)

class sVecUtls(object):
	pnts=[]
	name='cuad'
	def __init__(self, runmode, img):
		self.img = img
		if runmode == RUN_INTERACTIVE:
			self.showDialog()
		elif runmode == RUN_WITH_LAST_VALS:
			self.showDialog()
		elif runmode == RUN_NONINTERACTIVE:
			return

	def addRows(self, obj, table, col1,col2, row1,row2):
		table.attach(obj, col1, col2, row1, row2)
		obj.show()
		return obj

	def showDialog(self):
		self.dialog = gimpui.Dialog("Kreator kształtów", "rotdlg")
		self.dialog.set_position(gtk.WIN_POS_CENTER)
		self.table = gtk.Table(6, 8, True)
		self.table.set_homogeneous(False)
		self.table.set_row_spacings(2)
		self.table.set_col_spacings(2)
		self.table.show()
		row=1

		self.addRows(gtk.Label(_('Kształt:')), self.table, 0, 1, row, row+1)
		self.lnkPath=self.addRows(self.makeCombo(SHAPES), self.table, 1, 2, row, row+1)
		self.lnkPath.connect("changed", self.chPath)
		
		self.addRows(gtk.Label(_(' Wymiary zaznaczenia:')), self.table, 6, 8, row, row+1)
		row=row+1

		adj=gtk.Adjustment(0, 0, 100, 0.2)
		self.addRows(gtk.Label(_('Krągłość:')), self.table, 0, 1, row, row+1)
		self.cornerRad=self.addRows(gtk.SpinButton(adj, 0.0, 2), self.table, 1, 2, row, row+1)
		self.cornerRad.connect("output", self.chPath)
		self.addRows(gtk.Label(_('Szerokość:')), self.table, 6, 7, row, row+1)
		self.infoW=gtk.Label('?px')
		self.addRows(self.infoW, self.table, 7, 8, row, row+1)
		row=row+1

		self.addRows(gtk.Label(_('Grubość krzyża:')), self.table, 0, 1, row, row+1)
		self.crossWidth=self.addRows(gtk.SpinButton(gtk.Adjustment(0, 12, 600, 0.2), 0.0, 2), self.table, 1, 2, row, row+1)
		self.crossWidth.connect("output", self.chPath)
		self.addRows(gtk.Label(_('Wysokość:')), self.table, 6, 7, row, row+1)
		self.infoH=gtk.Label('?px')
		self.addRows(self.infoH, self.table, 7, 8, row, row+1)
		row=row+1


		lb=self.addRows(gtk.Label(_('Wartości dla kwadratów i prostokątów:')), self.table, 0, 4, row, row+1)
		lb.set_alignment(0.0,0.5)

		row=row+1
		
		self.addRows(gtk.Label(_('Narożniki:')), self.table, 0, 1, row, row+1)
		self.lnkPt=self.addRows(self.makeCombo(CORNERS), self.table, 1, 2, row, row+1)
		self.lnkPt.connect("changed", self.chPath)

		row=row+1

		lb=self.addRows(gtk.Label(_('Wartości dla wielokątów i gwiazd:')), self.table, 0, 4, row, row+1)
		lb.set_alignment(0.0,0.5)
		row=row+1

		adj=gtk.Adjustment(0, 3, 100, 1)
		self.addRows(gtk.Label(_('Wierzchołki:')), self.table, 0, 1, row, row+1)
		self.sidesNum=self.addRows(gtk.SpinButton(adj, 0.0, 0), self.table, 1, 2, row, row+1)
		self.sidesNum.connect("output", self.chPath)
		row=row+1

		adj=gtk.Adjustment(0, 10, 100, 1)
		self.addRows(gtk.Label(_('Drugi promień gwiazdy %:')), self.table, 0, 1, row, row+1)
		self.starRad2=self.addRows(gtk.SpinButton(adj, 0.0, 0), self.table, 1, 2, row, row+1)
		self.starRad2.connect("output", self.chPath)
		row=row+1

		self.addRows(gtk.Label(_('Obrót:')), self.table, 0, 1, row, row+1)
		self.shapeRot=self.addRows(gtk.SpinButton(gtk.Adjustment(0, -360, 360, 1), 0.0, 0), self.table, 1, 2, row, row+1)
		self.shapeRot.connect("output", self.chPath)
		row=row+1

		self.aa = gtk.DrawingArea()
		self.aa.set_size_request(200, 200)
		self.aa.set_events(gtk.gdk.EXPOSE)
		self.aaa = self.addRows(self.aa, self.table, 4, 6, 0,10)

		self.dialog.vbox.hbox1 = gtk.HBox(False, 7)
		self.dialog.vbox.hbox1.show()
		self.dialog.vbox.pack_start(self.dialog.vbox.hbox1, True, True, 7)
		self.dialog.vbox.hbox1.pack_start(self.table, True, True, 7)
		cancel_button = self.dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)

		ok_button = self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
		ok_button.connect("clicked", self.okbtn)
		self.dialog.show()
		self.dialog.run()
		self.chPath()

	def chPath(self, widget):
		lP=self.lnkPath.get_active()
		corners=self.lnkPt.get_active()
		rad=self.cornerRad.get_value()
		sides=self.sidesNum.get_value()
		starrad2=self.starRad2.get_value()
		non_empty, x1, y1, x2, y2 = pdb.gimp_selection_bounds(self.img)

		self.sidesNum.set_sensitive(not lP in [0,1,2])
		#self.cornerRad.set_sensitive(lP in [0,1])
		self.lnkPt.set_sensitive(lP in [0,1,2])
		self.starRad2.set_sensitive(not lP in [0,1,2])
		if non_empty==False:
			x1,y1,x2,y2=(0,0,self.img.width,self.img.height)
		an=x2 - x1
		al=y2 - y1
		self.infoW.set_text(str(an)+'px')
		self.infoH.set_text(str(al)+'px')
		la=an if an<al else al
		self.cornerRad.set_adjustment(gtk.Adjustment(rad, 0, int(la/2), 0.2))
		incX = (an-la)/2.0
		incY = (al-la)/2.0
		varBez=c = 0.551915024494
		iX,iY =(x1+incX,y1+incY)
		# por defecto un cuadrado
		bezs=[]
		pnts=[]
		p1=pnt2d(iX     ,iY)
		p2=pnt2d(iX + la,iY)
		p3=pnt2d(iX + la,iY + la)
		p4=pnt2d(iX     ,iY + la)
		bezs=[bezpnt(p1),bezpnt(p2),bezpnt(p3),bezpnt(p4)]
		an2=an/2.0
		al2=al/2.0
		rad2=rad/2.0
		name=_('square')
		if lP==0:
			# cuad fixed points
			fpts=[None, pnt2d(iX,iY+rad), pnt2d(iX+rad,iY), pnt2d(iX+la-rad,iY), pnt2d(iX+la,iY+rad), pnt2d(iX+la,iY+la-rad), pnt2d(iX+la-rad,iY+la), pnt2d(iX+rad,iY+la), pnt2d(iX,iY+la-rad)]
			if corners in [1,2]:
				name = name + _('Chamfer') if corners==1 else name + _('Rounded')
				r2=rad if corners==1 else rad*varBez
				fpts2=[pnt2d(iX,iY+r2) ,pnt2d(iX+r2,iY), pnt2d(iX+la-r2,iY), pnt2d(iX+la,iY+r2), pnt2d(iX+la,iY+la-r2), pnt2d(iX+la-r2,iY+la), pnt2d(iX+r2,iY+la), pnt2d(iX,iY+la-r2)]
			elif corners==3:
				name = name + _('RoundedInv')
				r2 = rad * varBez
				fpts2=[pnt2d(iX+r2,iY+rad), pnt2d(iX+rad,iY+r2) ,pnt2d(iX+la-rad,iY+r2), pnt2d(iX+la-r2,iY+rad), pnt2d(iX+la-r2,iY+la-rad), pnt2d(iX+la-rad,iY+la-r2), pnt2d(iX+rad,iY+la-r2), pnt2d(iX+r2,iY+la-rad)]
			if corners in [1,2,3]:
				bezs=[
					bezpnt(fpts[1],None,fpts2[0]), bezpnt(fpts[2],fpts2[1]),
					bezpnt(fpts[3],None,fpts2[2]), bezpnt(fpts[4],fpts2[3]),
					bezpnt(fpts[5],None,fpts2[4]), bezpnt(fpts[6],fpts2[5]),
					bezpnt(fpts[7],None,fpts2[6]), bezpnt(fpts[8],fpts2[7])]
		elif lP==1:
			name=_('rect')
			bezs=[bezpnt(pnt2d(x1,y1)), bezpnt(pnt2d(x1,y2)), bezpnt(pnt2d(x2,y2)), bezpnt(pnt2d(x2,y1))]
			if corners==3:
				name=name+_('RoundedInv')
				r2=rad*varBez
				fpts2=[pnt2d(x1+r2,y1+rad), pnt2d(x1+rad,y1+r2), pnt2d(x2-rad,y1+r2), pnt2d(x2-r2,y1+rad), pnt2d(x2-r2,y2-rad), pnt2d(x2-rad,y2-r2), pnt2d(x1+rad,y2-r2), pnt2d(x1+r2,y2-rad)]
			elif corners in [1,2]:
				name=name+_('Chamfer') if corners==1 else name+_('Rounded')
				r2=rad if corners==1 else rad*varBez
				fpts2=[pnt2d(x1,y1+r2), pnt2d(x1+r2,y1), pnt2d(x2-r2,y1), pnt2d(x2,y1+r2), pnt2d(x2,y2-r2), pnt2d(x2-r2,y2), pnt2d(x1+r2,y2), pnt2d(x1,y2-r2)]
			if corners in [1,2,3]:
				bezs=[
					bezpnt(pnt2d(x1,y1+rad),None,fpts2[0]), bezpnt(pnt2d(x1+rad,y1),fpts2[1]),
					bezpnt(pnt2d(x2-rad,y1),None,fpts2[2]), bezpnt(pnt2d(x2,y1+rad),fpts2[3]),
					bezpnt(pnt2d(x2,y2-rad),None,fpts2[4]), bezpnt(pnt2d(x2-rad,y2),fpts2[5]),
					bezpnt(pnt2d(x1+rad,y2),None,fpts2[6]), bezpnt(pnt2d(x1,y2-rad),fpts2[7])
				]
		elif lP==2:
			name=_('cross')
			crW = self.crossWidth.get_value()
			crw2 = crW/2
			pts=[pnt2d(x1+an2-crw2,y1), pnt2d(x1+an2+crw2,y1), pnt2d(x1+an2+crw2,y1+al2-crw2), pnt2d(x2, y1+al2-crw2),
				pnt2d(x2, y1+al2+crw2), pnt2d(x1+an2+crw2,y1+al2+crw2), pnt2d(x1+an2+crw2,y2), pnt2d(x1+an2-crw2,y2), 
				pnt2d(x1+an2-crw2,y1+al2+crw2), pnt2d(x1,y1+al2+crw2), pnt2d(x1,y1+al2-crw2), pnt2d(x1+an2-crw2,y1+al2-crw2)]
			bezs=[bezpnt(p) for p in pts]
			# chamfer
			if corners==1:
				name+=_('Chamfer')
				pts=[pnt2d(x1+an2-crw2+rad,y1), pnt2d(x1+an2+crw2-rad, y1), pnt2d(x1+an2+crw2, y1+rad), pnt2d(x1+an2+crw2, y1+al2-crw2),
					pnt2d(x2-rad,y1+al2-crw2), pnt2d(x2,y1+al2-crw2+rad), pnt2d(x2,y1+al2+crw2-rad), pnt2d(x2-rad,y1+al2+crw2),
					pnt2d(x1+an2+crw2, y1+al2+crw2), pnt2d(x1+an2+crw2, y2-rad), pnt2d(x1+an2+crw2-rad, y2),
					pnt2d(x1+an2-crw2+rad, y2),
					pnt2d(x1+an2-crw2, y2-rad),
					pnt2d(x1+an2-crw2, y1+al2+crw2),
					pnt2d(x1+rad,y1+al2+crw2),
					pnt2d(x1,y1+al2+crw2-rad),
					pnt2d(x1,y1+al2-crw2+rad),
					pnt2d(x1+rad,y1+al2-crw2),
					pnt2d(x1+an2-crw2, y1+al2-crw2),
					pnt2d(x1+an2-crw2, y1+rad),
				]
				bezs=[bezpnt(p) for p in pts]
			elif corners==2 and rad>0:
				r2=rad*varBez
				name=name+_('Rounded')
				bezs=[
					bezpnt(pnt2d(x1+an2-crw2+rad,y1),pnt2d(x1+an2-crw2+r2,y1)),
					bezpnt(pnt2d(x1+an2+crw2-rad,y1),None,pnt2d(x1+an2+crw2-r2, y1)),
					bezpnt(pnt2d(x1+an2+crw2,y1+rad),pnt2d(x1+an2+crw2, y1+rad-r2) ),
					bezpnt(pnt2d(x1+an2+crw2,y1+al2-crw2)),
					bezpnt(pnt2d(x2-rad,y1+al2-crw2),None,pnt2d(x2-rad+r2,y1+al2-crw2)),
					bezpnt(pnt2d(x2,y1+al2-crw2+rad),pnt2d(x2,y1+al2-crw2+rad-r2)),
					bezpnt(pnt2d(x2,y1+al2+crw2-rad),None,pnt2d(x2,y1+al2+crw2-rad+r2)),
					bezpnt(pnt2d(x2-rad,y1+al2+crw2),pnt2d(x2-rad+r2,y1+al2+crw2)),
					bezpnt(pnt2d(x1+an2+crw2,y1+al2+crw2)),
					bezpnt(pnt2d(x1+an2+crw2,y2-rad),None,pnt2d(x1+an2+crw2, y2-rad+r2)),
					bezpnt(pnt2d(x1+an2+crw2-rad, y2),pnt2d(x1+an2+crw2-rad+r2, y2)),
					bezpnt(pnt2d(x1+an2-crw2+rad, y2),None,pnt2d(x1+an2-crw2+rad-r2, y2)),
					bezpnt(pnt2d(x1+an2-crw2,y2-rad),pnt2d(x1+an2-crw2, y2-rad+r2)),
					bezpnt(pnt2d(x1+an2-crw2,y1+al2+crw2)),
					bezpnt(pnt2d(x1+rad,y1+al2+crw2),None,pnt2d(x1+rad-r2,y1+al2+crw2)),
					bezpnt(pnt2d(x1,y1+al2+crw2-rad),pnt2d(x1,y1+al2+crw2-rad+r2)),
					bezpnt(pnt2d(x1,y1+al2-crw2+rad),None,pnt2d(x1,y1+al2-crw2+rad-r2)),
					bezpnt(pnt2d(x1+rad,y1+al2-crw2),pnt2d(x1+rad-r2,y1+al2-crw2)),
					bezpnt(pnt2d(x1+an2-crw2, y1+al2-crw2)),
					bezpnt(pnt2d(x1+an2-crw2, y1+rad),None,pnt2d(x1+an2-crw2, y1+rad-r2))
				]
			elif corners==3 and rad>0:
				r2=rad*varBez
				name=name+_('RoundedInv')
				bezs=[
					bezpnt(pnt2d(x1+an2-crw2+rad,y1),pnt2d(x1+an2-crw2+rad,y1+r2)),
					bezpnt(pnt2d(x1+an2+crw2-rad,y1),None,pnt2d(x1+an2+crw2-rad, y1+r2)),
					bezpnt(pnt2d(x1+an2+crw2,y1+rad),pnt2d(x1+an2+crw2-r2, y1+rad) ),
					bezpnt(pnt2d(x1+an2+crw2,y1+al2-crw2)),
					bezpnt(pnt2d(x2-rad,y1+al2-crw2) ,None,pnt2d(x2-rad,y1+al2-crw2+r2)),
					bezpnt(pnt2d(x2,y1+al2-crw2+rad),pnt2d(x2-r2,y1+al2-crw2+rad)),
					bezpnt(pnt2d(x2,y1+al2+crw2-rad),None,pnt2d(x2-r2,y1+al2+crw2-rad)),
					bezpnt(pnt2d(x2-rad,y1+al2+crw2),pnt2d(x2-rad,y1+al2+crw2-r2)),
					bezpnt(pnt2d(x1+an2+crw2,y1+al2+crw2)),
					bezpnt(pnt2d(x1+an2+crw2,y2-rad),None,pnt2d(x1+an2+crw2-r2,y2-rad)),
					bezpnt(pnt2d(x1+an2+crw2-rad, y2),pnt2d(x1+an2+crw2-rad, y2-r2)),
					bezpnt(pnt2d(x1+an2-crw2+rad, y2),None,pnt2d(x1+an2-crw2+rad, y2-r2)),
					bezpnt(pnt2d(x1+an2-crw2, y2-rad),pnt2d(x1+an2-crw2+r2, y2-rad)),
					bezpnt(pnt2d(x1+an2-crw2, y1+al2+crw2)),
					bezpnt(pnt2d(x1+rad,y1+al2+crw2),None,pnt2d(x1+rad,y1+al2+crw2-r2)),
					bezpnt(pnt2d(x1,y1+al2+crw2-rad),pnt2d(x1+r2,y1+al2+crw2-rad)),
					bezpnt(pnt2d(x1,y1+al2-crw2+rad),None,pnt2d(x1+r2,y1+al2-crw2+rad)),
					bezpnt(pnt2d(x1+rad,y1+al2-crw2),pnt2d(x1+rad,y1+al2-crw2+r2)),
					bezpnt(pnt2d(x1+an2-crw2, y1+al2-crw2)),
					bezpnt(pnt2d(x1+an2-crw2, y1+rad),None,pnt2d(x1+an2-crw2+r2, y1+rad))
			]
		elif lP==3:
			name=_('rhomboid')
			pts=[pnt2d(x1+an2,y1), pnt2d(x1,y1+al2), pnt2d(x1+an2,y2), pnt2d(x2,y1+al2)]
			bezs=[bezpnt(pts[0]), bezpnt(pts[1]), bezpnt(pts[2]), bezpnt(pts[3])]
			# chamfer
			if corners==1:
				name+=_('Chamfer')
				bezs=[bezpnt(pnt2d(x1+an2-rad2,y1)),bezpnt(pnt2d(x1+an2+rad2,y1)),bezpnt(pnt2d(x2,y1+al2-rad2)), bezpnt(pnt2d(x2,y1+al2+rad2)), bezpnt(pnt2d(x1+an2+rad2,y2)), bezpnt(pnt2d(x1+an2-rad2,y2)), bezpnt(pnt2d(x1,y1+al2+rad2)), bezpnt(pnt2d(x1,y1+al2-rad2))]
			elif corners in [2,3] and rad>0:
				name=name+_('Rounded')
				bezs=[]
				arc1=arcLinesBez(pnt2d(0,an2),pnt2d(-al2,0),pnt2d(al2,0),rad)
				arc2=arcLinesBez(pnt2d(0,al2),pnt2d(-an2,0),pnt2d(an2,0),rad)
				bezs.extend(rotArcBez(arc1,90)+rotArcBez(arc2,0)+rotArcBez(arc1,270)+rotArcBez(arc2,180))
				for bez in bezs: bez.translate(x1+an/2.0, y1+al/2.0)
		elif lP in [4,5]:
			name=('Polygon') if lP==4 else ('Star')
			name = name+str(int(sides))
			if lP == 5: sides=sides*2
			rot=360/sides
			rd=la/2
			starr2=100/starrad2
			if lP==4: bezs = polyRoundBez(rd,rot,sides,rad)
			if lP==5: bezs = starRoundBez(rd,rd/starr2,rot,sides,rad)
			for bez in bezs: bez.translate(x1+an/2.0, y1+al/2.0)
		
		gp = gimppath(bezs)
		rotShape=self.shapeRot.get_value()
		if rotShape<>0: gp.rotate(math.radians(rotShape),x1+an/2.0,y1+al/2.0)
		gp.compact()
		pnts = gp.toGimpPath()

		self.pnts = pnts
		self.name = name

		self.style = self.aa.get_style()
		self.gc = self.style.fg_gc[gtk.STATE_NORMAL]

		w,h=(self.img.width, self.img.height)
		rel=w if w>h else h
		rel = rel * 1.0
		w2=int(w/rel*200)
		h2=int(h/rel*200)
		w=self.aa.window
		gc0 = w.new_gc()
		cmap = gc0.get_colormap()
		wgtl = widget.get_toplevel()
		bgColor = wgtl.style.bg[gtk.STATE_NORMAL]
		self.gc.set_foreground(bgColor)
		w.draw_rectangle(self.gc, True, 0, 0, w2, h2)
		self.gc.set_foreground(gtk.gdk.Color(0,0,0))
		self.gc.set_line_attributes(1, gtk.gdk.LINE_ON_OFF_DASH, gtk.gdk.CAP_ROUND, gtk.gdk.JOIN_ROUND)
		w.draw_polygon(self.gc,False,[(0,0), (w2-1,0), (w2-1,h2-1), (0,h2-1)])
		self.gc.set_line_attributes(1, gtk.gdk.LINE_SOLID, gtk.gdk.CAP_ROUND, gtk.gdk.JOIN_ROUND)
		p=[]
		for n in range(gp.len()-1):
			b1, b2 = (gp.getBezAt(n), gp.getBezAt(n+1))
			p=p+self.drawBez2([b1.Fixed().xy(),b1.Next().xy(),b2.Prev().xy(),b2.Fixed().xy()],self.gc,w,rel)
		b1 = gp.getBezAt(0)
		b2 = gp.getBezAt(gp.len()-1)
		p = p + self.drawBez2([b2.Fixed().xy() ,b2.Next().xy(),b1.Prev().xy() ,b1.Fixed().xy()], self.gc,w,rel)
		w.draw_polygon(self.gc, False, p)

	def drawBez2(self,pS,gc,w,rel):
		steps = 5
		pts=[]
		if ((pS[0]==pS[1]) and (pS[2]==pS[3])):
			pts=[(int(pS[0][0]/rel*200),int(pS[0][1]/rel*200))]
		else:
			oldPoint = pS[0]
			for point in bezier_curve_range(steps, pS):
				oldPoint = point
				pts=pts+[(int(point[0]/rel*200) , int(point[1]/rel*200))]
		return pts

	def makeCombo(self, Store):
		st = gtk.ListStore(str, int)
		for n in Store: st.append(n)
		self.cmb = gtk.ComboBox(st)
		cell = gtk.CellRendererText()
		self.cmb.pack_start(cell, True)
		self.cmb.add_attribute(cell, 'text', 0)
		self.cmb.set_entry_text_column(0)
		self.cmb.set_active(0)
		return self.cmb

	def okbtn(self, widget):
		v = pdb.gimp_vectors_new(self.img, self.name)
		pp=pdb.gimp_image_add_vectors(self.img, v, 0)
		pdb.gimp_vectors_stroke_new_from_points(v, 0, len(self.pnts), self.pnts, True)
		pdb.gimp_item_set_visible(v, True)

class pyWSutils(gimpplugin.plugin):
	def start(self):
		gimp.main(self.init, self.quit, self.query, self._run)
	def init(self):
		pass
	def quit(self):
		pass
	def query(self):
		author = "jfgarcia"
		cright = "jfgarcia"
		date = "2013"
		plug_descr = _("Tworzenie różnych kształtów ścieżek na środku bieżącego zaznaczenia")
		plug_params = [(PDB_INT32, "run_mode", "Run mode"), (PDB_IMAGE, "image", "Input image"),]
		gimp.install_procedure("path_shape_creator3", plug_descr, plug_descr, author, cright, date, _("<Image>/Filters/Path/Kreator kształtów ścieżek..."), "RGB*, GRAY*", PLUGIN, plug_params,[])

	def path_shape_creator3(self, runmode, img):
		sVecUtls(runmode, img)

if __name__ == '__main__':
	pyWSutils().start()
# 409